/*
 * Copyright (C) 2016 Red Hat, Inc.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package io.syndesis.server.endpoint.v1.state;

import com.fasterxml.jackson.annotation.JsonProperty;
import com.google.errorprone.annotations.CanIgnoreReturnValue;
import com.google.errorprone.annotations.Var;
import edu.umd.cs.findbugs.annotations.SuppressFBWarnings;
import java.util.Objects;
import java.util.Set;
import javax.annotation.CheckReturnValue;
import javax.annotation.Nullable;
import javax.annotation.ParametersAreNonnullByDefault;
import javax.annotation.concurrent.Immutable;
import javax.annotation.concurrent.NotThreadSafe;
import javax.validation.ConstraintViolation;
import javax.validation.ConstraintViolationException;
import javax.validation.Validation;
import javax.validation.Validator;
import org.immutables.value.Generated;

/**
 * Immutable implementation of {@link ClientSideStateTest.Data}.
 * <p>
 * Use the builder to create immutable instances:
 * {@code ImmutableData.builder()}.
 * Use the static factory method to create immutable instances:
 * {@code ImmutableData.of()}.
 */
@Generated(from = "ClientSideStateTest.Data", generator = "Immutables")
@SuppressWarnings({"all"})
@SuppressFBWarnings
@ParametersAreNonnullByDefault
@javax.annotation.Generated("org.immutables.processor.ProxyProcessor")
@Immutable
@CheckReturnValue
final class ImmutableData implements ClientSideStateTest.Data {
  private final int integer;
  private final String string;

  private ImmutableData(int integer, String string) {
    this.integer = integer;
    this.string = string;
  }

  /**
   * @return The value of the {@code integer} attribute
   */
  @JsonProperty("integer")
  @Override
  public int getInteger() {
    return integer;
  }

  /**
   * @return The value of the {@code string} attribute
   */
  @JsonProperty("string")
  @Override
  public String getString() {
    return string;
  }

  /**
   * Copy the current immutable object by setting a value for the {@link ClientSideStateTest.Data#getInteger() integer} attribute.
   * A value equality check is used to prevent copying of the same value by returning {@code this}.
   * @param value A new value for integer
   * @return A modified copy of the {@code this} object
   */
  public final ImmutableData withInteger(int value) {
    if (this.integer == value) return this;
    return validate(new ImmutableData(value, this.string));
  }

  /**
   * Copy the current immutable object by setting a value for the {@link ClientSideStateTest.Data#getString() string} attribute.
   * An equals check used to prevent copying of the same value by returning {@code this}.
   * @param value A new value for string (can be {@code null})
   * @return A modified copy of the {@code this} object
   */
  public final ImmutableData withString(String value) {
    if (Objects.equals(this.string, value)) return this;
    return validate(new ImmutableData(this.integer, value));
  }

  /**
   * This instance is equal to all instances of {@code ImmutableData} that have equal attribute values.
   * @return {@code true} if {@code this} is equal to {@code another} instance
   */
  @Override
  public boolean equals(@Nullable Object another) {
    if (this == another) return true;
    return another instanceof ImmutableData
        && equalTo((ImmutableData) another);
  }

  private boolean equalTo(ImmutableData another) {
    return integer == another.integer
        && Objects.equals(string, another.string);
  }

  /**
   * Computes a hash code from attributes: {@code integer}, {@code string}.
   * @return hashCode value
   */
  @Override
  public int hashCode() {
    @Var int h = 5381;
    h += (h << 5) + integer;
    h += (h << 5) + Objects.hashCode(string);
    return h;
  }

  /**
   * Prints the immutable value {@code Data} with attribute values.
   * @return A string representation of the value
   */
  @Override
  public String toString() {
    return "Data{"
        + "integer=" + integer
        + ", string=" + string
        + "}";
  }

  /**
   * Construct a new immutable {@code Data} instance.
   * @param integer The value for the {@code integer} attribute
   * @param string The value for the {@code string} attribute
   * @return An immutable Data instance
   */
  public static ClientSideStateTest.Data of(int integer, String string) {
    return validate(new ImmutableData(integer, string));
  }

  private static final Validator validator = Validation.buildDefaultValidatorFactory().getValidator();

  private static ImmutableData validate(ImmutableData instance) {
    Set<ConstraintViolation<ImmutableData>> constraintViolations = validator.validate(instance);
    if (!constraintViolations.isEmpty()) {
      throw new ConstraintViolationException(constraintViolations);
    }
    return instance;
  }

  /**
   * Creates an immutable copy of a {@link ClientSideStateTest.Data} value.
   * Uses accessors to get values to initialize the new immutable instance.
   * If an instance is already immutable, it is returned as is.
   * @param instance The instance to copy
   * @return A copied immutable Data instance
   */
  public static ClientSideStateTest.Data copyOf(ClientSideStateTest.Data instance) {
    if (instance instanceof ImmutableData) {
      return (ImmutableData) instance;
    }
    return ImmutableData.builder()
        .createFrom(instance)
        .build();
  }

  /**
   * Creates a builder for {@link ClientSideStateTest.Data Data}.
   * <pre>
   * ImmutableData.builder()
   *    .integer(int) // optional {@link ClientSideStateTest.Data#getInteger() integer}
   *    .string(String | null) // nullable {@link ClientSideStateTest.Data#getString() string}
   *    .build();
   * </pre>
   * @return A new Data builder
   */
  public static ImmutableData.Builder builder() {
    return new ImmutableData.Builder();
  }

  /**
   * Builds instances of type {@link ClientSideStateTest.Data Data}.
   * Initialize attributes and then invoke the {@link #build()} method to create an
   * immutable instance.
   * <p><em>{@code Builder} is not thread-safe and generally should not be stored in a field or collection,
   * but instead used immediately to create instances.</em>
   */
  @Generated(from = "ClientSideStateTest.Data", generator = "Immutables")
  @NotThreadSafe
  public static final class Builder {
    private int integer;
    private @Nullable String string;

    private Builder() {
    }

    /**
     * Fill a builder with attribute values from the provided {@code Data} instance.
     * Regular attribute values will be replaced with those from the given instance.
     * Absent optional values will not replace present values.
     * @param instance The instance from which to copy values
     * @return {@code this} builder for use in a chained invocation
     */
    @CanIgnoreReturnValue 
    public final Builder createFrom(ClientSideStateTest.Data instance) {
      Objects.requireNonNull(instance, "instance");
      integer(instance.getInteger());
      String stringValue = instance.getString();
      if (stringValue != null) {
        string(stringValue);
      }
      return this;
    }

    /**
     * Initializes the value for the {@link ClientSideStateTest.Data#getInteger() integer} attribute.
     * @param integer The value for integer 
     * @return {@code this} builder for use in a chained invocation
     */
    @CanIgnoreReturnValue 
    @JsonProperty("integer")
    public final Builder integer(int integer) {
      this.integer = integer;
      return this;
    }

    /**
     * Initializes the value for the {@link ClientSideStateTest.Data#getString() string} attribute.
     * @param string The value for string (can be {@code null})
     * @return {@code this} builder for use in a chained invocation
     */
    @CanIgnoreReturnValue 
    @JsonProperty("string")
    public final Builder string(String string) {
      this.string = string;
      return this;
    }

    /**
     * Builds a new {@link ClientSideStateTest.Data Data}.
     * @return An immutable instance of Data
     * @throws java.lang.IllegalStateException if any required attributes are missing
     */
    public ClientSideStateTest.Data build() {
      return ImmutableData.validate(new ImmutableData(integer, string));
    }
  }
}
